using Serilog.Events;

namespace PresenterConsole;

internal static class Program
{
  public static void Main(string[] parArgs)
  {
    var engine = new Engine.EngineBuilder()
      .Headless()
      .LogToFile(true, "log.txt")
      .LogLevel(LogEventLevel.Debug)
      .Presenter(parEngine => new ConsolePresenter(parEngine))
      .InputHandler(_ => new ConsoleInputHandler())
      .AssetFolder(Path.GetFullPath("../DoomDeathmatch/asset"))
      .Build();

    DoomDeathmatch.DoomDeathmatch.Initialize(engine);

    engine.Run();
  }
}